#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels

#define OLED_RESET -1 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C ///< OLED I2C address for 128x32 display
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

int trig = 11; // Trigger pin of the distance sensor
int echo = 12; // Echo pin of the distance sensor

void setup() {
  // Initialize serial communication
  Serial.begin(9600);

  // Initialize distance sensor pins
  pinMode(trig, OUTPUT);
  pinMode(echo, INPUT);

  // Initialize the OLED display
  if (!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("SSD1306 allocation failed"));
    for (;;); // Don't proceed, loop forever
  }

  // Initial OLED display setup
  display.clearDisplay();
  displayFace(":)"); // Show initial smiley face
  delay(2000);
}

void loop() {
  // Measure distance using the sensor
  long distance = getDistance();

  // Clear the OLED display
  display.clearDisplay();

  // Determine which face to display based on distance
  if (distance <= 25) {
    displayFace(":)"); // Display smiley face if distance <= 25 inches
  } else {
    displayFace(":("); // Display sad face if distance > 25 inches
  }

  delay(500);
}

long getDistance() {
  // Trigger the sensor
  digitalWrite(trig, LOW);
  delayMicroseconds(2);
  digitalWrite(trig, HIGH);
  delayMicroseconds(10);
  digitalWrite(trig, LOW);

  // Measure the echo duration
  long duration = pulseIn(echo, HIGH);

  // Convert the duration to distance in inches
  long distance = duration / 74 / 2;

  // Print the distance to the Serial Monitor (for debugging)
  Serial.print("Distance: ");
  Serial.print(distance);
  Serial.println(" inches");

  return distance;
}

void displayFace(const char *face) {
  // Draw a circle in the center of the screen
  int centerX = SCREEN_WIDTH / 2;
  int centerY = SCREEN_HEIGHT / 2;
  int radius = 12;

  display.drawCircle(centerX, centerY, radius, SSD1306_WHITE);

  // Display the face inside the circle
  display.setTextSize(1);
  display.setTextColor(SSD1306_WHITE);
  int textWidth = 12; // Adjust for the approximate width of ":)" or ":("
  display.setCursor(centerX - textWidth / 2, centerY - 4); // Center text horizontally and vertically
  display.print(face);

  // Update the OLED
  display.display();
}
